/* evilwm - Minimalist Window Manager for X
 * Copyright (C) 1999-2000 Ciaran Anscomb <kja3@atdot.org>
 * see README for license and other details. */

#include "evilwm.h"
#include <stdlib.h>
#include <X11/Xatom.h>

void handle_key_event(XKeyEvent *e) {
	Client *c = find_client(e->window);

	if (!c) {
		if (!current)
			return;
		c = current;
	}
	switch (XKeycodeToKeysym(dpy,e->keycode,0)) {
		case XK_h: case XK_j: case XK_k: case XK_l:
		case XK_y: case XK_u: case XK_b: case XK_n:
			XPutBackEvent(dpy, (XEvent *)e); move(c); break;
		case XK_Return:
			system(opt_term ? opt_term : DEF_TERM "&"); break;
		case XK_Escape:
			send_wm_delete(c); break;
		case XK_Tab:
			next(c); break;
		case XK_Insert: case XK_KP_Insert:
			XLowerWindow(dpy, c->parent); break;
		case XK_1: case XK_2: case XK_3: case XK_4:
		case XK_5: case XK_6: case XK_7: case XK_8:
			switch_vdesk(XKeycodeToKeysym(dpy,e->keycode,0)-XK_1);
			break;
		case XK_f:
			if (c->vdesk == -1) {
				c->vdesk = vdesk;
				XSetWindowBackground(dpy, c->parent, fg.pixel);
				XClearWindow(dpy, c->parent);
			} else {
				c->vdesk = -1;
				XSetWindowBackground(dpy, c->parent, fc.pixel);
				XClearWindow(dpy, c->parent);
			}
			break;
	}
}

void handle_button_event(XButtonEvent *e)
{
	Client *c = find_client(e->window);

	/* if (e->window == root) {
		switch (e->button) {
			case Button1: system(opt_term ? opt_term : DEF_TERM "&"); break;
			case Button3: quit_nicely(); break;
		}
	} else if (c) { */
	if (c && e->window != root) {
		switch (e->button) {
			case Button1:
				move(c); break;
			case Button2:
				resize(c); break;
			case Button3:
				XLowerWindow(dpy, c->parent); break;
		}
	}
}

void handle_configure_request(XConfigureRequestEvent *e)
{
	Client *c = find_client(e->window);
	XWindowChanges wc;

	if (c) {
		ungravitate(c);
		if (e->value_mask & CWX) c->x = e->x;
		if (e->value_mask & CWY) c->y = e->y;
		if (e->value_mask & CWWidth) c->width = e->width;
		if (e->value_mask & CWHeight) c->height = e->height;
		gravitate(c);

		wc.x = c->x - c->border;
		wc.y = c->y - c->border;
		wc.width = c->width + (c->border*2);
		wc.height = c->height + (c->border*2);
		wc.border_width = opt_bw;
		wc.sibling = e->above;
		wc.stack_mode = e->detail;
		XConfigureWindow(dpy, c->parent, e->value_mask, &wc);
		send_config(c);
	}

	wc.x = c ? c->border : e->x;
	wc.y = c ? c->border : e->y;
	wc.width = e->width;
	wc.height = e->height;
	wc.sibling = e->above;
	wc.stack_mode = e->detail;
	XConfigureWindow(dpy, e->window, e->value_mask, &wc);
}

void handle_map_request(XMapRequestEvent *e)
{
	Client *c = find_client(e->window);

	if (c) {
		XMapWindow(dpy, c->window);
		XMapRaised(dpy, c->parent);
		set_wm_state(c, NormalState);
	} else
		make_new_client(e->window);
}

void handle_unmap_event(XUnmapEvent *e)
{
	Client *c = find_client(e->window);

	if (c) {
		if (c->ignore_unmap) c->ignore_unmap--;
		else remove_client(c, 0);
	}
}

void handle_client_message(XClientMessageEvent *e)
{
	/*Client *c = find_client(e->window);

	if (c && e->message_type == xa_wm_change_state &&
		e->format == 32 && e->data.l[0] == IconicState) hide(c);
	*/
}

void handle_colormap_change(XColormapEvent *e)
{
	Client *c = find_client(e->window);

	if (c && e->new) {
		c->cmap = e->colormap;
		XInstallColormap(dpy, c->cmap);
	}
}

void handle_property_change(XPropertyEvent *e)
{
	Client *c = find_client(e->window);
	long dummy;

	if (c) {
		switch (e->atom) {
			case XA_WM_NAME:
				if (c->name) XFree(c->name);
				XFetchName(dpy, c->window, &c->name);
				XClearWindow(dpy, c->parent);
				redraw(c);
				break;
			case XA_WM_NORMAL_HINTS:
				XGetWMNormalHints(dpy, c->window, c->size, &dummy);
		}
	}
}

void handle_enter_event(XCrossingEvent *e)
{
	Client *c = find_client(e->window);
	if (c) {
		XInstallColormap(dpy, c->cmap);
		XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
		if (current && c!=current) {
			XSetWindowBackground(dpy, current->parent, bg.pixel);
			XClearWindow(dpy, current->parent);
		}
		XSetWindowBackground(dpy, c->parent, (c->vdesk==-1)?fc.pixel:fg.pixel);
		XClearWindow(dpy, c->parent);
		current = c;
		XGrabButton(dpy, AnyButton, Mod1Mask, c->parent, False, ButtonMask,
		GrabModeAsync, GrabModeSync, None, None);
	}
}

void handle_expose_event(XExposeEvent *e)
{
	Client *c = find_client(e->window);

	if (c && e->count == 0) redraw(c);
}

/* void handle_reparent_event(XReparentEvent *e)
{
		Client *c = find_client(e->window);
		Client *dest = find_client(e->parent);

		if (c && !dest) remove_client(c, 0);
} */
