#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

/* default settings */

#define DEF_FONT	"lucidasans-10"
#define DEF_FG		"goldenrod"
#define DEF_BG		"grey50"
#define DEF_FC		"blue"
#define DEF_TERM	"xterm"
#define DEF_BW		0
#define SPACE		3
#define MINSIZE		15

/* some coding shorthand */

#define ChildMask	(SubstructureRedirectMask|SubstructureNotifyMask)
#define ButtonMask	(ButtonPressMask|ButtonReleaseMask)
#define MouseMask	(ButtonMask|PointerMotionMask)

#define grab(w, mask, curs) \
	(XGrabPointer(dpy, w, False, mask, GrabModeAsync, GrabModeAsync, \
	None, curs, CurrentTime) == GrabSuccess)
#define ungrab() \
	XUngrabServer(dpy); XUngrabPointer(dpy, CurrentTime)
#define setmouse(w, x, y) \
	XWarpPointer(dpy, None, w, 0, 0, 0, 0, x, y)
#define gravitate(c) \
	change_gravity(c, 1)
#define ungravitate(c) \
	change_gravity(c, -1)

/* client structure */

typedef struct Client Client;

struct Client {
	Client	*next;
	Window	window;
	Window	parent;
	Window	trans;

	char	*name;

	Colormap	cmap;
	XSizeHints	*size;
	int		ignore_unmap;

	int		x;
	int		y;
	int		width;
	int		height;
	int		border;
	int		vdesk;
};

/* declarations for global variables in main.c */

extern Display		*dpy;
extern Client		*current;
extern int		screen;
extern Window		root;
extern GC		invert_gc;
extern XFontStruct	*font;
extern int		iheight;
extern XColor		fg, bg, fc;
extern Client		*head_client;
extern Atom		xa_wm_state;
extern Atom		xa_wm_change_state;
extern Atom		xa_wm_protos;
extern Atom		xa_wm_delete;
extern Atom		xa_wm_cmapwins;
extern Cursor		move_curs;
extern Cursor		resize_curs;
extern char		*opt_display;
extern char		*opt_font;
extern char		*opt_fg;
extern char		*opt_bg;
extern char		*opt_fc;
extern char		*opt_term;
extern int		opt_bw;
extern int		vdesk;

/* client.c */

Client *find_client(Window w);
int send_xmessage(Window w, Atom a, long x);
int wm_state(Client *c);
void change_gravity(Client *c, int multiplier);
void redraw(Client *c);
void remove_client(Client *c, int from_cleanup);
void send_config(Client *c);
void send_wm_delete(Client *c);
void set_shape(Client *c);
void set_wm_state(Client *c, int state);
void client_to_head(Client *c);

/* events.c */

void handle_key_event(XKeyEvent *e);
void handle_button_event(XButtonEvent *e);
void handle_client_message(XClientMessageEvent *e);
void handle_colormap_change(XColormapEvent *e);
void handle_configure_request(XConfigureRequestEvent *e);
void handle_enter_event(XCrossingEvent *e);
void handle_expose_event(XExposeEvent *e);
void handle_map_request(XMapRequestEvent *e);
void handle_property_change(XPropertyEvent *e);
/* void handle_reparent_event(XReparentEvent *e); */
void handle_unmap_event(XUnmapEvent *e);

/* main.c */

int main(int argc, char *argv[]);
void scan_windows();
void setup_display();

/* misc.c */

int handle_xerror(Display *dpy, XErrorEvent *e);
void do_event_loop();
void dump_clients();
void err(char *fmt, ...);
void quit_nicely();
void show_event(XEvent e);

/* new.c */

void init_position(Client *c);
void initialize(Client *c, Window w);
void make_new_client(Window w);
void reparent(Client *c);

/* screen.c */

void drag(Client *c);
void draw_outline(Client *c);
void get_mouse_position(int *x, int *y);
void hide(Client *c);
void move(Client *c);
void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2);
void resize(Client *c);
void sweep(Client *c);
void unhide(Client *c);
void next(Client *c);
void switch_vdesk(int v);
